@@ -2,7 +2,7 @@
 /* vim:set et sts=4: */
 /* ibus - The Input Bus
  * Copyright (C) 2008-2013 Peng Huang <shawn.p.huang@gmail.com>
- * Copyright (C) 2018-2023 Takao Fujiwara <takao.fujiwara1@gmail.com>
+ * Copyright (C) 2018-2024 Takao Fujiwara <takao.fujiwara1@gmail.com>
  * Copyright (C) 2008-2021 Red Hat, Inc.
  *
  * This library is free software; you can redistribute it and/or
@@ -322,6 +322,7 @@ static const guint IBUS_MODIFIER_FILTER =
         IBUS_HYPER_MASK |
         IBUS_META_MASK);
 
+
 static void
 ibus_engine_class_init (IBusEngineClass *class)
 {
@@ -1115,7 +1116,17 @@ ibus_engine_filter_key_event (IBusEngine *engine,
     g_return_val_if_fail (IBUS_IS_ENGINE (engine), FALSE);
 
     priv = engine->priv;
-    modifiers = state & IBUS_MODIFIER_FILTER;
+    modifiers = state;
+    /*
+     * GTK3 has both IBUS_SUPER_MASK & IBUS_MOD4_MASK.
+     * GTK4 has IBUS_SUPER_MASK.
+     * Qt5 has IBUS_MOD4_MASK.
+     */
+    if (modifiers & IBUS_SUPER_MASK) {
+        modifiers &= ~IBUS_SUPER_MASK;
+        modifiers |= IBUS_MOD4_MASK;
+    }
+    modifiers = modifiers & IBUS_MODIFIER_FILTER;
     if (keyval >= IBUS_KEY_A && keyval <= IBUS_KEY_Z &&
         (modifiers & IBUS_SHIFT_MASK) != 0) {
         keyval = keyval - IBUS_KEY_A + IBUS_KEY_a;
@@ -1137,6 +1148,7 @@ ibus_engine_filter_key_event (IBusEngine *engine,
                 keys->state == modifiers &&
                 (keys->keycode == 0 || keys->keycode == keycode)) {
                 ibus_engine_panel_extension (engine, name);
+                g_list_free (names);
                 return TRUE;
             }
         }
